/*
 * Example Code for the Paper
 *
 * "Mesh Saliency via Local Curvature Entropy", Eurographics 2016
 *
 * by M. Limper, A. Kuijper and D. Fellner
 *
 */

#ifndef LCE_MESHUTILS_H
#define LCE_MESHUTILS_H

#include "TriMesh.h"

namespace lce
{

//------------------------------------------------------------------------------------------------------------------

enum TransferFunction
{
    RAINBOW,
    HEAT
};

//------------------------------------------------------------------------------------------------------------------

inline float triangleArea3D(const OpenMesh::Vec3f & p0, const OpenMesh::Vec3f & p1, const OpenMesh::Vec3f & p2)
{
    return ((p0-p1) % (p0-p2)).norm() * 0.5f;
}

//------------------------------------------------------------------------------------------------------------------

inline float triangleArea2D(const OpenMesh::Vec2f & p0, const OpenMesh::Vec2f & p1, const OpenMesh::Vec2f & p2)
{
    return ((p1[0]-p0[0])*(p2[1]-p0[1])-(p2[0]-p0[0])*(p1[1]-p0[1])) * 0.5f;
}

//------------------------------------------------------------------------------------------------------------------

void mapScalarToColor(OpenMesh::Vec3f & rgbColor, float value, TransferFunction transferFunction = HEAT);

//------------------------------------------------------------------------------------------------------------------

void getGeodesicNeighborhood(const TriMesh & mesh, TriMesh::VertexHandle v, float radius, OpenMesh::EPropHandleT<float> & edgeLengthProperty,
                                      std::vector<TriMesh::VertexHandle> & neighbors);

//------------------------------------------------------------------------------------------------------------------

void getKNeighborhood(const TriMesh & mesh, TriMesh::VertexHandle v, unsigned k, std::set<TriMesh::VertexHandle> & neighbors);

//------------------------------------------------------------------------------------------------------------------

void intersectLines2D(const OpenMesh::Vec2f & p0, const OpenMesh::Vec2f & d0,
                      const OpenMesh::Vec2f & p1, const OpenMesh::Vec2f & d1,
                      OpenMesh::Vec2f & intersection);

//------------------------------------------------------------------------------------------------------------------

float surfaceArea(TriMesh & mesh);

//------------------------------------------------------------------------------------------------------------------

float faceArea(TriMesh & mesh, TriMesh::FaceHandle & face);

//------------------------------------------------------------------------------------------------------------------

template<typename T>
T angle(const OpenMesh::VectorT<T, 3> & v, const OpenMesh::VectorT<T, 3> & w)
{
    float nv = v.norm();
    float nw = w.norm();

    if (nv < FLT_EPSILON || nw < FLT_EPSILON)
        return 0.0;

    float q = (v|w) / (nv * nw);

    return q >= 1.0 ? 0.0 : (q <= -1.0 ? M_PI : acos(q));
}

//------------------------------------------------------------------------------------------------------------------

OpenMesh::EPropHandleT<float> computeEdgeLengths(TriMesh & mesh);

//------------------------------------------------------------------------------------------------------------------

OpenMesh::HPropHandleT<float> computeWedgeVoronoiAreas(TriMesh & mesh);

//------------------------------------------------------------------------------------------------------------------

OpenMesh::VPropHandleT<float> computeVertexAreas(TriMesh & mesh);

//------------------------------------------------------------------------------------------------------------------

} //namespace lce


#endif //LCE_MESHUTILS_H
